import axios from 'axios';
import { get } from 'lodash';
/**
 * Tries to retrieve an endpoint from the site root configuration file
 * Every eva-suite hosted deployment can have a config file which details which
 * backend server to use.
 * Development installation will often not have this file to fallback on a
 * selection mechanism from the customer manager
 *
 * @returns {(Promise<string|undefined>)}
 */
export const getEndpointFromRootConfig = async () => {
    // There is a special EVA config loader for pre-application availability of the
    // config for analytics and such
    // We can check its existence here to prevent duplicate retrieval
    //
    const evaConfigLoaderEndpoint = get(window, 'eva_config_loader.config.endpoint');
    if (evaConfigLoaderEndpoint) {
        return evaConfigLoaderEndpoint;
    }
    const evaConfigLoaderStatus = get(window, 'eva_config_loader.status');
    if (evaConfigLoaderStatus === 'NOT_FOUND') {
        return;
    }
    // If this is a customer specific deployment we will
    // be able to get the URL from the config.json on / (root level)
    //
    const configFileUrl = '/eva-config.json';
    console.log(`[EVA-CONTEXT] fetch config info: ${configFileUrl}`);
    try {
        const buildInfoResponse = await axios.get(configFileUrl, { responseType: 'json' });
        const evaConfigInfo = buildInfoResponse.data;
        console.log(`[EVA-CONTEXT] fetched config info: ${evaConfigInfo.endpoint}`);
        return evaConfigInfo.endpoint;
    }
    catch (fetchConfigInfoError) {
        console.warn('[EVA-CONTEXT] failed to fetch config info for app', fetchConfigInfoError);
    }
};
//# sourceMappingURL=config.js.map