import { get, isNil, isEqual, slice, uniq, concat, find, isArray as isArray$1 } from 'lodash';
import { v4 } from 'uuid';
import { createStoreObservable, createServiceReducer, createServiceRequestLogic, StoreBuilder, createEndpointAction, createCurrentUserUpdateAction, createServiceRequestAction, createServiceResponseAction, createServiceResetAction } from '@springtree/eva-sdk-redux-store-builder';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { BehaviorSubject } from 'rxjs';
import axios from 'axios';
import { firebase } from '@firebase/app';
import '@firebase/auth';
import '@firebase/firestore';
import { evaResponseError } from '@springtree/eva-sdk-core-response-error';
import { setCoreSetting } from '@springtree/eva-sdk-core-settings';
import { evaWarningLogger } from '@springtree/eva-sdk-core-warning';

function __async(g){return new Promise(function(s,j){function c(a,x){try{var r=g[x?"throw":"next"](a);}catch(e){j(e);return}r.done?s(r.value):Promise.resolve(r.value).then(c,d);}function d(e){c(e,1);}c();})}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function isFunction(x) {
    return typeof x === 'function';
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var _enable_super_gross_mode_that_will_cause_bad_things = false;
var config = {
    Promise: undefined,
    set useDeprecatedSynchronousErrorHandling(value) {
        if (value) {
            var error = /*@__PURE__*/ new Error();
            /*@__PURE__*/ console.warn('DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n' + error.stack);
        }
        _enable_super_gross_mode_that_will_cause_bad_things = value;
    },
    get useDeprecatedSynchronousErrorHandling() {
        return _enable_super_gross_mode_that_will_cause_bad_things;
    },
};

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function hostReportError(err) {
    setTimeout(function () { throw err; }, 0);
}

/** PURE_IMPORTS_START _config,_util_hostReportError PURE_IMPORTS_END */
var empty = {
    closed: true,
    next: function (value) { },
    error: function (err) {
        if (config.useDeprecatedSynchronousErrorHandling) {
            throw err;
        }
        else {
            hostReportError(err);
        }
    },
    complete: function () { }
};

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var isArray = /*@__PURE__*/ (function () { return Array.isArray || (function (x) { return x && typeof x.length === 'number'; }); })();

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function isObject(x) {
    return x !== null && typeof x === 'object';
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var UnsubscriptionErrorImpl = /*@__PURE__*/ (function () {
    function UnsubscriptionErrorImpl(errors) {
        Error.call(this);
        this.message = errors ?
            errors.length + " errors occurred during unsubscription:\n" + errors.map(function (err, i) { return i + 1 + ") " + err.toString(); }).join('\n  ') : '';
        this.name = 'UnsubscriptionError';
        this.errors = errors;
        return this;
    }
    UnsubscriptionErrorImpl.prototype = /*@__PURE__*/ Object.create(Error.prototype);
    return UnsubscriptionErrorImpl;
})();
var UnsubscriptionError = UnsubscriptionErrorImpl;

/** PURE_IMPORTS_START _util_isArray,_util_isObject,_util_isFunction,_util_UnsubscriptionError PURE_IMPORTS_END */
var Subscription = /*@__PURE__*/ (function () {
    function Subscription(unsubscribe) {
        this.closed = false;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (unsubscribe) {
            this._unsubscribe = unsubscribe;
        }
    }
    Subscription.prototype.unsubscribe = function () {
        var errors;
        if (this.closed) {
            return;
        }
        var _a = this, _parentOrParents = _a._parentOrParents, _unsubscribe = _a._unsubscribe, _subscriptions = _a._subscriptions;
        this.closed = true;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (_parentOrParents instanceof Subscription) {
            _parentOrParents.remove(this);
        }
        else if (_parentOrParents !== null) {
            for (var index = 0; index < _parentOrParents.length; ++index) {
                var parent_1 = _parentOrParents[index];
                parent_1.remove(this);
            }
        }
        if (isFunction(_unsubscribe)) {
            try {
                _unsubscribe.call(this);
            }
            catch (e) {
                errors = e instanceof UnsubscriptionError ? flattenUnsubscriptionErrors(e.errors) : [e];
            }
        }
        if (isArray(_subscriptions)) {
            var index = -1;
            var len = _subscriptions.length;
            while (++index < len) {
                var sub = _subscriptions[index];
                if (isObject(sub)) {
                    try {
                        sub.unsubscribe();
                    }
                    catch (e) {
                        errors = errors || [];
                        if (e instanceof UnsubscriptionError) {
                            errors = errors.concat(flattenUnsubscriptionErrors(e.errors));
                        }
                        else {
                            errors.push(e);
                        }
                    }
                }
            }
        }
        if (errors) {
            throw new UnsubscriptionError(errors);
        }
    };
    Subscription.prototype.add = function (teardown) {
        var subscription = teardown;
        if (!teardown) {
            return Subscription.EMPTY;
        }
        switch (typeof teardown) {
            case 'function':
                subscription = new Subscription(teardown);
            case 'object':
                if (subscription === this || subscription.closed || typeof subscription.unsubscribe !== 'function') {
                    return subscription;
                }
                else if (this.closed) {
                    subscription.unsubscribe();
                    return subscription;
                }
                else if (!(subscription instanceof Subscription)) {
                    var tmp = subscription;
                    subscription = new Subscription();
                    subscription._subscriptions = [tmp];
                }
                break;
            default: {
                throw new Error('unrecognized teardown ' + teardown + ' added to Subscription.');
            }
        }
        var _parentOrParents = subscription._parentOrParents;
        if (_parentOrParents === null) {
            subscription._parentOrParents = this;
        }
        else if (_parentOrParents instanceof Subscription) {
            if (_parentOrParents === this) {
                return subscription;
            }
            subscription._parentOrParents = [_parentOrParents, this];
        }
        else if (_parentOrParents.indexOf(this) === -1) {
            _parentOrParents.push(this);
        }
        else {
            return subscription;
        }
        var subscriptions = this._subscriptions;
        if (subscriptions === null) {
            this._subscriptions = [subscription];
        }
        else {
            subscriptions.push(subscription);
        }
        return subscription;
    };
    Subscription.prototype.remove = function (subscription) {
        var subscriptions = this._subscriptions;
        if (subscriptions) {
            var subscriptionIndex = subscriptions.indexOf(subscription);
            if (subscriptionIndex !== -1) {
                subscriptions.splice(subscriptionIndex, 1);
            }
        }
    };
    Subscription.EMPTY = (function (empty) {
        empty.closed = true;
        return empty;
    }(new Subscription()));
    return Subscription;
}());
function flattenUnsubscriptionErrors(errors) {
    return errors.reduce(function (errs, err) { return errs.concat((err instanceof UnsubscriptionError) ? err.errors : err); }, []);
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var rxSubscriber = /*@__PURE__*/ (function () {
    return typeof Symbol === 'function'
        ? /*@__PURE__*/ Symbol('rxSubscriber')
        : '@@rxSubscriber_' + /*@__PURE__*/ Math.random();
})();

/** PURE_IMPORTS_START tslib,_util_isFunction,_Observer,_Subscription,_internal_symbol_rxSubscriber,_config,_util_hostReportError PURE_IMPORTS_END */
var Subscriber = /*@__PURE__*/ (function (_super) {
    __extends(Subscriber, _super);
    function Subscriber(destinationOrNext, error, complete) {
        var _this = _super.call(this) || this;
        _this.syncErrorValue = null;
        _this.syncErrorThrown = false;
        _this.syncErrorThrowable = false;
        _this.isStopped = false;
        switch (arguments.length) {
            case 0:
                _this.destination = empty;
                break;
            case 1:
                if (!destinationOrNext) {
                    _this.destination = empty;
                    break;
                }
                if (typeof destinationOrNext === 'object') {
                    if (destinationOrNext instanceof Subscriber) {
                        _this.syncErrorThrowable = destinationOrNext.syncErrorThrowable;
                        _this.destination = destinationOrNext;
                        destinationOrNext.add(_this);
                    }
                    else {
                        _this.syncErrorThrowable = true;
                        _this.destination = new SafeSubscriber(_this, destinationOrNext);
                    }
                    break;
                }
            default:
                _this.syncErrorThrowable = true;
                _this.destination = new SafeSubscriber(_this, destinationOrNext, error, complete);
                break;
        }
        return _this;
    }
    Subscriber.prototype[rxSubscriber] = function () { return this; };
    Subscriber.create = function (next, error, complete) {
        var subscriber = new Subscriber(next, error, complete);
        subscriber.syncErrorThrowable = false;
        return subscriber;
    };
    Subscriber.prototype.next = function (value) {
        if (!this.isStopped) {
            this._next(value);
        }
    };
    Subscriber.prototype.error = function (err) {
        if (!this.isStopped) {
            this.isStopped = true;
            this._error(err);
        }
    };
    Subscriber.prototype.complete = function () {
        if (!this.isStopped) {
            this.isStopped = true;
            this._complete();
        }
    };
    Subscriber.prototype.unsubscribe = function () {
        if (this.closed) {
            return;
        }
        this.isStopped = true;
        _super.prototype.unsubscribe.call(this);
    };
    Subscriber.prototype._next = function (value) {
        this.destination.next(value);
    };
    Subscriber.prototype._error = function (err) {
        this.destination.error(err);
        this.unsubscribe();
    };
    Subscriber.prototype._complete = function () {
        this.destination.complete();
        this.unsubscribe();
    };
    Subscriber.prototype._unsubscribeAndRecycle = function () {
        var _parentOrParents = this._parentOrParents;
        this._parentOrParents = null;
        this.unsubscribe();
        this.closed = false;
        this.isStopped = false;
        this._parentOrParents = _parentOrParents;
        return this;
    };
    return Subscriber;
}(Subscription));
var SafeSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(SafeSubscriber, _super);
    function SafeSubscriber(_parentSubscriber, observerOrNext, error, complete) {
        var _this = _super.call(this) || this;
        _this._parentSubscriber = _parentSubscriber;
        var next;
        var context = _this;
        if (isFunction(observerOrNext)) {
            next = observerOrNext;
        }
        else if (observerOrNext) {
            next = observerOrNext.next;
            error = observerOrNext.error;
            complete = observerOrNext.complete;
            if (observerOrNext !== empty) {
                context = Object.create(observerOrNext);
                if (isFunction(context.unsubscribe)) {
                    _this.add(context.unsubscribe.bind(context));
                }
                context.unsubscribe = _this.unsubscribe.bind(_this);
            }
        }
        _this._context = context;
        _this._next = next;
        _this._error = error;
        _this._complete = complete;
        return _this;
    }
    SafeSubscriber.prototype.next = function (value) {
        if (!this.isStopped && this._next) {
            var _parentSubscriber = this._parentSubscriber;
            if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                this.__tryOrUnsub(this._next, value);
            }
            else if (this.__tryOrSetError(_parentSubscriber, this._next, value)) {
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.error = function (err) {
        if (!this.isStopped) {
            var _parentSubscriber = this._parentSubscriber;
            var useDeprecatedSynchronousErrorHandling = config.useDeprecatedSynchronousErrorHandling;
            if (this._error) {
                if (!useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(this._error, err);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, this._error, err);
                    this.unsubscribe();
                }
            }
            else if (!_parentSubscriber.syncErrorThrowable) {
                this.unsubscribe();
                if (useDeprecatedSynchronousErrorHandling) {
                    throw err;
                }
                hostReportError(err);
            }
            else {
                if (useDeprecatedSynchronousErrorHandling) {
                    _parentSubscriber.syncErrorValue = err;
                    _parentSubscriber.syncErrorThrown = true;
                }
                else {
                    hostReportError(err);
                }
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.complete = function () {
        var _this = this;
        if (!this.isStopped) {
            var _parentSubscriber = this._parentSubscriber;
            if (this._complete) {
                var wrappedComplete = function () { return _this._complete.call(_this._context); };
                if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(wrappedComplete);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, wrappedComplete);
                    this.unsubscribe();
                }
            }
            else {
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.__tryOrUnsub = function (fn, value) {
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            this.unsubscribe();
            if (config.useDeprecatedSynchronousErrorHandling) {
                throw err;
            }
            else {
                hostReportError(err);
            }
        }
    };
    SafeSubscriber.prototype.__tryOrSetError = function (parent, fn, value) {
        if (!config.useDeprecatedSynchronousErrorHandling) {
            throw new Error('bad call');
        }
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            if (config.useDeprecatedSynchronousErrorHandling) {
                parent.syncErrorValue = err;
                parent.syncErrorThrown = true;
                return true;
            }
            else {
                hostReportError(err);
                return true;
            }
        }
        return false;
    };
    SafeSubscriber.prototype._unsubscribe = function () {
        var _parentSubscriber = this._parentSubscriber;
        this._context = null;
        this._parentSubscriber = null;
        _parentSubscriber.unsubscribe();
    };
    return SafeSubscriber;
}(Subscriber));

/** PURE_IMPORTS_START tslib,_Subscriber PURE_IMPORTS_END */
function map(project, thisArg) {
    return function mapOperation(source) {
        if (typeof project !== 'function') {
            throw new TypeError('argument is not a function. Are you looking for `mapTo()`?');
        }
        return source.lift(new MapOperator(project, thisArg));
    };
}
var MapOperator = /*@__PURE__*/ (function () {
    function MapOperator(project, thisArg) {
        this.project = project;
        this.thisArg = thisArg;
    }
    MapOperator.prototype.call = function (subscriber, source) {
        return source.subscribe(new MapSubscriber(subscriber, this.project, this.thisArg));
    };
    return MapOperator;
}());
var MapSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(MapSubscriber, _super);
    function MapSubscriber(destination, project, thisArg) {
        var _this = _super.call(this, destination) || this;
        _this.project = project;
        _this.count = 0;
        _this.thisArg = thisArg || _this;
        return _this;
    }
    MapSubscriber.prototype._next = function (value) {
        var result;
        try {
            result = this.project.call(this.thisArg, value, this.count++);
        }
        catch (err) {
            this.destination.error(err);
            return;
        }
        this.destination.next(result);
    };
    return MapSubscriber;
}(Subscriber));

/** PURE_IMPORTS_START tslib,_Subscriber PURE_IMPORTS_END */
function distinctUntilChanged(compare, keySelector) {
    return function (source) { return source.lift(new DistinctUntilChangedOperator(compare, keySelector)); };
}
var DistinctUntilChangedOperator = /*@__PURE__*/ (function () {
    function DistinctUntilChangedOperator(compare, keySelector) {
        this.compare = compare;
        this.keySelector = keySelector;
    }
    DistinctUntilChangedOperator.prototype.call = function (subscriber, source) {
        return source.subscribe(new DistinctUntilChangedSubscriber(subscriber, this.compare, this.keySelector));
    };
    return DistinctUntilChangedOperator;
}());
var DistinctUntilChangedSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(DistinctUntilChangedSubscriber, _super);
    function DistinctUntilChangedSubscriber(destination, compare, keySelector) {
        var _this = _super.call(this, destination) || this;
        _this.keySelector = keySelector;
        _this.hasKey = false;
        if (typeof compare === 'function') {
            _this.compare = compare;
        }
        return _this;
    }
    DistinctUntilChangedSubscriber.prototype.compare = function (x, y) {
        return x === y;
    };
    DistinctUntilChangedSubscriber.prototype._next = function (value) {
        var key;
        try {
            var keySelector = this.keySelector;
            key = keySelector ? keySelector(value) : value;
        }
        catch (err) {
            return this.destination.error(err);
        }
        var result = false;
        if (this.hasKey) {
            try {
                var compare = this.compare;
                result = compare(this.key, key);
            }
            catch (err) {
                return this.destination.error(err);
            }
        }
        else {
            this.hasKey = true;
        }
        if (!result) {
            this.key = key;
            this.destination.next(value);
        }
    };
    return DistinctUntilChangedSubscriber;
}(Subscriber));

/** PURE_IMPORTS_START tslib,_Subscriber PURE_IMPORTS_END */
function filter(predicate, thisArg) {
    return function filterOperatorFunction(source) {
        return source.lift(new FilterOperator(predicate, thisArg));
    };
}
var FilterOperator = /*@__PURE__*/ (function () {
    function FilterOperator(predicate, thisArg) {
        this.predicate = predicate;
        this.thisArg = thisArg;
    }
    FilterOperator.prototype.call = function (subscriber, source) {
        return source.subscribe(new FilterSubscriber(subscriber, this.predicate, this.thisArg));
    };
    return FilterOperator;
}());
var FilterSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(FilterSubscriber, _super);
    function FilterSubscriber(destination, predicate, thisArg) {
        var _this = _super.call(this, destination) || this;
        _this.predicate = predicate;
        _this.thisArg = thisArg;
        _this.count = 0;
        return _this;
    }
    FilterSubscriber.prototype._next = function (value) {
        var result;
        try {
            result = this.predicate.call(this.thisArg, value, this.count++);
        }
        catch (err) {
            this.destination.error(err);
            return;
        }
        if (result) {
            this.destination.next(value);
        }
    };
    return FilterSubscriber;
}(Subscriber));

/**
 * Helper method to create an observable for a path within the Redux store
 * Will only trigger when value set and has changed since previous emit
 *
 * @export
 * @template SVC The service the path points to
 * @template STORE The store shape to pull the service from
 * @param {Observable<STORE>} store$ The observable store data
 * @param {string} path The path to the service
 * @returns {Observable<SVC>}
 */
function createObservableForPath(store$, path) {
    return store$.pipe(
    // tap( value => logger.debug('[EVA-CONTEXT]: State update', value)),
    map((value) => get(value, path)), filter(value => !isNil(value)), distinctUntilChanged(isEqual));
}

class PreferenceManager {
    /**
     * Creates an instance of PreferenceManager
     *
     * @param {Store<STORE>} store The redux store which we need for the current user name
     * @param {EvaStorage<STORAGE>} storage The storage for the preferences
     */
    constructor(store, storage, storageKey = 'evaSuite:preferences', historyLimit = 10, storageEventEmitter = window) {
        this.store = store;
        this.storage = storage;
        this.storageKey = storageKey;
        this.historyLimit = historyLimit;
        /**
         * The current in-memory copy of the preferences
         *
         * @private
         * @type {IPreferences}
         */
        this._preferences = {};
        // Subscribe to storage events to update the preferences
        //
        try {
            storageEventEmitter.addEventListener('storage', (storageEvent) => {
                if (storageEvent.key && storageEvent.key === storageKey) {
                    console.log(`[EVA-CONTEXT] new preferences from storage: ${storageEvent.newValue}`);
                    // Update in-memory preferences from storage
                    //
                    try {
                        const newValue = JSON.parse(storageEvent.newValue);
                        this.preferences = newValue;
                    }
                    catch (error) {
                        console.warn('[EVA-CONTEXT] failed to parse new preferences', error);
                    }
                }
            });
        }
        catch (storageEventListenerError) {
            console.warn('[EVA-CONTEXT] Failed to register storage event listener', storageEventListenerError);
        }
        // Get the current username from the store
        //
        let currentUsername = this.getCurrentUserName();
        // Create the behavior subject for the preferences and populate it for the current user
        //
        this.preferences$ = new BehaviorSubject(this.preferences[currentUsername]);
        // Subscribe to getCurrentUser changes to switch the preferences if a new
        // user logs in or out
        //
        store.subscribe(() => {
            const newUserName = this.getCurrentUserName();
            if (newUserName !== currentUsername) {
                // Change to new username and retrieve or initialize the new preferences
                //
                currentUsername = newUserName;
                const preferences = this.getCurrentPreferences(currentUsername);
                // Use the internal setter to update the whole object because this will
                // both update storage and emit on the observable
                //
                this.preferences = preferences;
            }
        });
    }
    /**
     * Setter to capture changes and emit them on the observable
     *
     * @private
     */
    set preferences(preferences) {
        if (!isEqual(preferences, this.preferences)) {
            this._preferences = preferences;
            // Update preferences in storage
            //
            this.storage.setItem(this.storageKey, JSON.stringify(this._preferences));
            // Emit the new user preferences
            //
            const currentUsername = this.getCurrentUserName();
            this.preferences$.next(this._preferences[currentUsername]);
        }
    }
    /**
     * Helper method to create empty initial preferences for Anonymous
     *
     * @private
     * @returns {IPreferences}
     */
    createDefaultPreferences() {
        const defaultPreferences = {
            history: {
                DiscountID: [],
                InvoiceID: [],
                OrderID: [],
                OrganizationUnitID: [],
                ProductID: [],
                SearchQuery: [],
                UserID: [],
            },
            settings: {},
        };
        return defaultPreferences;
    }
    /**
     * Internal getter to access the preferences and return a default empty object
     * if not initialized
     *
     * @private
     */
    get preferences() {
        return this._preferences || { Anonymous: this.createDefaultPreferences() };
    }
    /**
     * Helper method to determine the current username based on the redux store
     *
     * @private
     * @returns {string}
     */
    getCurrentUserName() {
        var _a, _b;
        const storeState = this.store.getState();
        const currentUser = storeState.currentUser;
        const currentUsername = ((_b = (_a = currentUser) === null || _a === void 0 ? void 0 : _a.data.response) === null || _b === void 0 ? void 0 : _b.User.EmailAddress) || 'Anonymous';
        return currentUsername;
    }
    /**
     * Helper method to get the current preferences
     * Can optionally be initialized for the current user
     *
     * @private
     * @param {string} [currentUserName] The username to initialize for
     * @returns {IPreferences}
     */
    getCurrentPreferences(currentUsername) {
        // Fetch the preferences object and current username
        //
        const preferences = this.preferences;
        // Initialize user preferences if required
        //
        if (currentUsername && !preferences[currentUsername]) {
            preferences[currentUsername] = this.createDefaultPreferences();
        }
        return preferences;
    }
    /**
     * This will add a recent item ID to the history preferences for the current user
     *
     * @param {keyof IRecentItems} key
     * @param {number} id
     */
    addRecentItem(key, id) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Push the id in the recent history
        // We will use some lodash magic to keep the history at 10 unique items
        //
        const currentItems = preferences[currentUsername].history[key];
        const newItems = slice(uniq(concat(currentItems, [id])), 0, this.historyLimit);
        preferences[currentUsername].history[key] = newItems;
        // Use the internal setter to update the whole object because this will
        // both update storage and emit on the observable
        //
        this.preferences = preferences;
    }
    /**
     * This will retrieve recent item IDs for the current user
     *
     * @param {keyof IRecentItems} key
     */
    getRecentItems(key) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Return a copy not the original
        //
        const currentItems = preferences[currentUsername].history[key];
        return JSON.parse(JSON.stringify(currentItems));
    }
    /**
     * This will add an application setting
     *
     * @param {string} appName The name of the application the setting is for
     * @param {string} key The name of the setting
     * @param {*} value The value of the setting
     */
    addAppSetting(appName, key, value) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Safety check that the value can be stringified
        //
        try {
            JSON.stringify(value);
        }
        catch (error) {
            console.warn(`[EVA-CONTEXT] Invalid application setting supplied for ${key}`, error);
            throw new Error(`Invalid application setting supplied for ${key}`);
        }
        if (!preferences[currentUsername].settings[appName]) {
            preferences[currentUsername].settings[appName] = {};
        }
        // Set the application setting
        //
        preferences[currentUsername].settings[appName][key] = value;
        // Use the internal setter to update the whole object because this will
        // both update storage and emit on the observable
        //
        this.preferences = preferences;
    }
    /**
     * Returns the current settings for an application.
     * More exact typings can be cast by the calling application
     *
     * @param {string} appName
     * @returns {{[key: string]: any}}
     */
    getAppSettings(appName) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        return preferences[currentUsername].settings[appName] || {};
    }
}

/**
 * The session manager is used to create and manage session for the React Context.
 *
 * @export
 * @class EvaSessionManager
 */
class EvaSessionManager {
    constructor() {
        /**
         * The collection of EVA sessions that can be swapped into and out of the React context
         *
         * @private
         * @type {{[key:string]: IEvaSession =}
         */
        this.sessions = [];
    }
    /**
     * Creates a new EVA Context session by creating the Redux store and bootstrapping it
     * Caller can supply a session identifier or one will be created automatically
     *
     * @param {IEvaSessionOptions} options
     * @returns {Promise<IEvaSession>}
     */
    createSession(builder, options) {return __async(function*(){
        // Use provided session ID or generate a new one
        //
        const sessionId = options.sessionId || v4();
        // Create the Redux store and its observable
        //
        const store = yield builder.createStore();
        const store$ = createStoreObservable(store);
        // Create the preference manager and observable
        //
        const preferences = new PreferenceManager(store, options.storage);
        const session = {
            sessionId,
            preferences,
            preferences$: preferences.preferences$,
            store: store,
            store$: store$,
            storage: options.storage,
            currentUserData$: createObservableForPath(store$, 'currentUser.data.response'),
            currentUserStatus$: createObservableForPath(store$, 'currentUserStatus'),
            currentUserToken$: createObservableForPath(store$, 'currentUserToken'),
            getEvaEndpoint: () => __async(function*(){
                const endPointUrl = store.getState().endpointUrl;
                const endPoint = yield bootstrapEndpoint({ uri: endPointUrl });
                return endPoint;
            }()),
        };
        this.sessions.push(session);
        // Activate the session if requested which is the default
        //
        if (!options.createOnly) {
            this.currentSession = session;
            // Update session ID in local storage
            //
            if (options.storage) {
                options.storage.setItem('evaSuite:sessionId', session.sessionId);
            }
        }
        return session;
    }.call(this))}
    /**
     * Retrieves a session from the pool by it's session identifier
     *
     * @param {string} sessionId
     * @returns {(IEvaSession|undefined)}
     */
    getSession(sessionId) {
        return find(this.sessions, { sessionId });
    }
}

/**
 * Tries to retrieve an endpoint from the site root configuration file
 * Every eva-suite hosted deployment can have a config file which details which
 * backend server to use.
 * Development installation will often not have this file to fallback on a
 * selection mechanism from the customer manager
 *
 * @returns {(Promise<string|undefined>)}
 */
const getEndpointFromRootConfig = () => __async(function*(){
    // There is a special EVA config loader for pre-application availability of the
    // config for analytics and such
    // We can check its existence here to prevent duplicate retrieval
    //
    const evaConfigLoaderEndpoint = get(window, 'eva_config_loader.config.endpoint');
    if (evaConfigLoaderEndpoint) {
        return evaConfigLoaderEndpoint;
    }
    const evaConfigLoaderStatus = get(window, 'eva_config_loader.status');
    if (evaConfigLoaderStatus === 'NOT_FOUND') {
        return;
    }
    // If this is a customer specific deployment we will
    // be able to get the URL from the config.json on / (root level)
    //
    const configFileUrl = '/eva-config.json';
    console.log(`[EVA-CONTEXT] fetch config info: ${configFileUrl}`);
    try {
        const buildInfoResponse = yield axios.get(configFileUrl, { responseType: 'json' });
        const evaConfigInfo = buildInfoResponse.data;
        console.log(`[EVA-CONTEXT] fetched config info: ${evaConfigInfo.endpoint}`);
        return evaConfigInfo.endpoint;
    }
    catch (fetchConfigInfoError) {
        console.warn('[EVA-CONTEXT] failed to fetch config info for app', fetchConfigInfoError);
    }
}());

/**
 * Use EvaLogin widget to ask user to select an OAuth provider
 * login in and then we receive the firebase authentication session
 *
 * @param {springtree_solutionsEvaSuiteUiEvaLogin} widget The login widget instance to use for user interaction
 */
const authenticateWithOAuthProviders = (options) => {
    const authentication = firebase.auth();
    const loginWithGoogle = {
        id: 'google',
        label: 'Login with Google',
        data: {},
    };
    const loginWithMicrosoft = {
        id: 'microsoft',
        label: 'Login with Microsoft',
        data: {},
    };
    const loginWithGithub = {
        id: 'github',
        label: 'Login with Github',
        data: {},
    };
    options.loginWidget.showChoice({
        question: 'How do you want to authenticate?',
        label: 'Choose authentication provider',
        answers: [loginWithGoogle, loginWithMicrosoft, loginWithGithub],
    });
    // Listen for the users' choice
    //
    options.loginWidget.once('answer', (answer) => __async(function*(){
        options.loginWidget.close();
        switch (answer.id) {
            case loginWithGoogle.id:
                yield authentication.signInWithPopup(new firebase.auth.GoogleAuthProvider());
                break;
            case loginWithMicrosoft.id:
                yield authentication.signInWithPopup(new firebase.auth.OAuthProvider('microsoft.com'));
                break;
            case loginWithGithub.id:
                yield authentication.signInWithPopup(new firebase.auth.GithubAuthProvider());
                break;
            default:
                alert('Unknown authentication provider');
                break;
        }
    }()));
};
/**
 * Tries to retrieve an endpoint from the EVA customer manager
 * This will present a number of choices to the user and requires an OAuth login
 *
 * @returns {(Promise<string|undefined>)}
 */
const getEndpointFromEvaCustomerManager = (options) => {
    return new Promise((resolve, reject) => {
        firebase.initializeApp(options.firebaseConfig);
        const authentication = firebase.auth();
        if (!authentication) {
            throw new Error('Missing authorisation support');
        }
        // Listen to Firebase authentication changes
        //
        const firebaseAuthenticationListener = authentication.onAuthStateChanged((user) => __async(function*(){
            if (user) {
                // Stop listening to Firebase authentication changes
                //
                firebaseAuthenticationListener();
                const db = firebase.firestore();
                // Select all available customers from FireStore
                //
                const customers = yield db.collection('customer').get();
                // Ask user to choose a customer
                //
                options.loginWidget.showChoice({
                    question: 'Select EVA customer',
                    label: 'EVA Customer',
                    answers: customers.docs.map((item) => {
                        const data = item.data();
                        return {
                            data,
                            id: data.name,
                            label: data.name,
                        };
                    }),
                });
                // Listen for the answer of the customer selector
                //
                options.loginWidget.once('answer', (answer) => __async(function*(){
                    const name = answer.label;
                    const environments = get(answer, 'data.baseEnvironments', []);
                    // If there is only one environment
                    // Select the only environment available
                    //
                    if (environments.length === 1) {
                        options.loginWidget.close();
                        resolve(environments[0].endpoint);
                    }
                    else if (environments.length > 2) {
                        // Retrieve the full details of the base environments
                        //
                        const baseEnvPromises = environments.map((baseEnvRef) => {
                            return baseEnvRef.get().then(snapshot => snapshot.data());
                        });
                        const baseEnvironments = yield Promise.all(baseEnvPromises);
                        // Ask to select an environment
                        //
                        options.loginWidget.showChoice({
                            question: `Select ${name} environment`,
                            label: 'Environment',
                            answers: baseEnvironments.map((data) => {
                                return {
                                    id: data.type,
                                    label: data.type,
                                    data: data.endpoint,
                                };
                            }),
                        });
                        // Bootstrap the EVA SDK with the selected endpoint
                        //
                        options.loginWidget.once('answer', (answer) => __async(function*(){
                            options.loginWidget.close();
                            resolve(answer.data);
                        }()));
                    }
                }()));
            }
            else {
                authenticateWithOAuthProviders(options);
            }
        }()));
    });
};

// All global storage keys use this prefix
//
const STORAGE_PREFIX = 'evaSuite';

// Activate all the default service interceptors for Axios
//
// evaAvailableServices.activate();
evaResponseError.activate();
evaWarningLogger.activate();
class EvaContextBuilder {
    /**
     * Creates an instance of EvaContextBuilder
     * Provide the template classes for your storage and store state interfaces
     *
     * @param {IEvaContextBuilderOptions<STORAGE>} options The options needed to create the store
     */
    constructor(options) {
        this.options = options;
        /**
         * The collection of redux logics to build the store with
         *
         * @private
         * @type {Logic<STORE>[]}
         */
        this.logic = [];
        /**
         * The collection of reducers to build the store with
         *
         * @private
         * @type {ReducersMapObject<Partial<STORE>>}
         */
        this.reducer = {};
        // Set our application details
        // Calling application MUST provide these
        //
        setCoreSetting('appName', options.appName);
        setCoreSetting('appVersion', options.appVersion);
    }
    /**
     * Purge all previously set reducers and logics
     *
     */
    resetReducerAndLogic() {
        this.logic = [];
        this.reducer = {};
    }
    /**
     * Helper method to add both the reducer and logic for an EVA service in one call
     *
     * @template SVC
     * @param {keyof STORE} path
     * @param {new () => SVC} service
     */
    addEvaService(path, service) {
        // Check we are not overwriting a previous reducer
        //
        if (this.reducer[path]) {
            throw new Error(`Duplicate reducer provided for ${path}`);
        }
        this.reducer[path] = createServiceReducer(service, { stateKey: path });
        this.logic.push(...createServiceRequestLogic(service, { stateKey: path }));
    }
    /**
     * Add a custom state reducer to the store
     *
     * @param {keyof STORE} path
     * @param {Reducer<STORE[keyof STORE]>} reducer
     */
    addReducer(path, reducer) {
        this.reducer[path] = reducer;
    }
    /**
     * Add custom store logic
     *
     * @param {Logic<STORE>} logic The redux logic that is to be added to the store
     */
    addLogic(logic) {
        if (isArray$1(logic)) {
            this.logic.push(...logic);
        }
        else {
            this.logic.push(logic);
        }
    }
    /**
     * Create a Redux store using the provided and previously configured reducers and logics
     *
     */
    createStore() {return __async(function*(){
        console.info('[EVA-CONTEXT]: Bootstrapping EVA Store');
        const logicCount = this.logic.length;
        const reducerCount = Object.keys(this.reducer).length;
        console.info(`[EVA-CONTEXT]: Store with ${reducerCount} reducer(s) and ${logicCount} logic(s)`);
        // Create the default redux store with current user support
        // Also create the RxJS observable for it
        // Provide our storage instance to setup inter-app sync of user token en endpoint
        // If the endpoint and user token are present in local storage they will be populated in the store
        //
        const builder = new StoreBuilder();
        const store = builder.createStore({
            reducers: this.reducer,
            logics: this.logic,
            storage: this.options.storage,
            storageKeyPrefix: STORAGE_PREFIX,
        });
        // Figure out what the the endpoint needs to be
        // The procedure to do so is as follows:
        //
        // * check if an endpoint is set in the store
        // * check for an eva-config in the site root and use that
        // * use the EVA customer manager to select an EVA backend
        //
        console.info('[EVA-CONTEXT]: Checking EVA endpoint...');
        if (!store.getState().endpointUrl) {
            // No endpoint has been set for the current store
            // Time to fetch and check the EVA config
            //
            const rootConfigEndpoint = yield getEndpointFromRootConfig();
            if (rootConfigEndpoint) {
                console.info('[EVA-CONTEXT]: Configure endpoint from site config', rootConfigEndpoint);
                const rootEndpointAction = createEndpointAction({ endpoint: rootConfigEndpoint });
                store.dispatch(rootEndpointAction);
            }
            else {
                // No EVA config could be found in the site root
                // We will get an environment selection from the EVA customer manager
                // The customer manager is a developer tool which requires a valid login
                // with an OAuth account
                //
                const customerManagerEndpoint = yield getEndpointFromEvaCustomerManager(this.options);
                if (customerManagerEndpoint) {
                    console.info('[EVA-CONTEXT]: Configure endpoint from customer manager', customerManagerEndpoint);
                    const customerManagerEndpointAction = createEndpointAction({ endpoint: customerManagerEndpoint });
                    store.dispatch(customerManagerEndpointAction);
                }
                else {
                    throw new Error('Failed to initialise the application. Missing endpoint');
                }
            }
        }
        // We will have an endpoint url if we get here so let's bootstrap it
        //
        const endpointUrl = store.getState().endpointUrl;
        console.info('[EVA-CONTEXT]: Bootstrapping endpoint...', endpointUrl);
        yield bootstrapEndpoint({ uri: endpointUrl, storage: this.options.storage });
        // Fetch anonymous or actual user depending on what was pulled out of storage
        //
        const updateUserAction = createCurrentUserUpdateAction();
        store.dispatch(updateUserAction);
        try {
            yield updateUserAction.payload.promise;
        }
        catch (error) {
            console.error('[EVA-CONTEXT]: Error while fetching current user', error);
        }
        // We're done here, the store is usable
        //
        return store;
    }.call(this))}
}

/**
 * Helper method to create an EVA Service context based on the Redux store
 *
 * @export
 * @template SVC
 * @param {keyof IEvaAdminStore} path
 * @param {IEvaSession} session
 * @returns {IEvaServiceContext<SVC>}
 */
function createServiceContext(params) {
    const context = {
        getState: () => get(params.session.store.getState(), params.path),
        getState$: () => createObservableForPath(params.session.store$, `${params.path})`),
        getRequest$: () => createObservableForPath(params.session.store$, `${params.path}.data.request`),
        getResponse$: () => createObservableForPath(params.session.store$, `${params.path}.data.response`),
        isFetching$: () => createObservableForPath(params.session.store$, `${params.path}.fetching`),
        getError$: () => createObservableForPath(params.session.store$, `${params.path}.error`),
        fetch: (payload = params.defaultPayload) => {
            // Create and dispatch the REQUEST action for the service on the current
            // store for the EVA session
            //
            const action = createServiceRequestAction({
                payload,
                service: params.service,
                stateOptions: { stateKey: params.path },
            });
            params.session.store.dispatch(action);
            return action;
        },
        response: (payload) => {
            // Create and dispatch the RESPONSE action for the service on the current
            // store for the EVA session
            // This might be useful for manually updating a response if it has been
            // supplied from another source.
            //
            const action = createServiceResponseAction({
                data: payload,
                service: params.service,
                stateOptions: { stateKey: params.path },
                serviceContext: 'MANUAL',
                chainContext: 'MANUAL',
            });
            params.session.store.dispatch(action);
            return action;
        },
        reset: (newState) => {
            // Create and dispatch the RESET action for the service on the current
            // store for the EVA session
            // You can optionally supply what you want the new state to be if not empty
            //
            const action = createServiceResetAction({
                state: newState,
                stateKey: params.path,
            });
            params.session.store.dispatch(action);
            return action;
        },
        call: (payload, options) => __async(function*(){
            const endPoint = yield params.session.getEvaEndpoint();
            return yield endPoint.callService(params.service, payload, options);
        }()),
    };
    return context;
}

export { EvaContextBuilder, EvaSessionManager, STORAGE_PREFIX, createObservableForPath, createServiceContext, getEndpointFromEvaCustomerManager, getEndpointFromRootConfig };
//# sourceMappingURL=eva-suite-bootstrap.es5.js.map
