import { IEvaSession } from './session';
import { EvaContextBuilder } from '../store';
import { IEvaReduxStoreDefaultShape } from '@springtree/eva-sdk-redux-store-builder';
import { EvaStorage } from '@springtree/eva-sdk-core-storage';
import { IEvaSuiteStorage } from '../utils/storage';
/**
 * Options to create an EVA Context session
 * Uses the store bootstrap options as a baseline
 *
 * @export
 * @interface IEvaSessionOptions
 * @extends {IEvaBootstrapOptions}
 */
export interface IEvaSessionOptions<STORAGE extends IEvaSuiteStorage> {
    /**
     * You can optionally supply a session id. Will auto-generate a uuid based
     * one if omitted
     *
     * @type {string}
     */
    sessionId?: string;
    /**
     * Set this flag to create the session but not activate
     * The default is to always activate the newly created session
     *
     * @type {boolean}
     */
    createOnly?: boolean;
    /**
     * The EVA SDK based storage provider
     *
     * @type {EvaStorage<STORAGE>}
     */
    storage: EvaStorage<STORAGE>;
}
/**
 * The session manager is used to create and manage session for the React Context.
 *
 * @export
 * @class EvaSessionManager
 */
export declare class EvaSessionManager<STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage> {
    /**
     * The collection of EVA sessions that can be swapped into and out of the React context
     *
     * @private
     * @type {{[key:string]: IEvaSession =}
     */
    private sessions;
    /**
     * The currently active and selected EVA Session
     *
     * @private
     * @type {(IEvaSession|undefined)}
     */
    currentSession: IEvaSession<STORE, STORAGE> | undefined;
    /**
     * Creates a new EVA Context session by creating the Redux store and bootstrapping it
     * Caller can supply a session identifier or one will be created automatically
     *
     * @param {IEvaSessionOptions} options
     * @returns {Promise<IEvaSession>}
     */
    createSession(builder: EvaContextBuilder<STORE, STORAGE>, options: IEvaSessionOptions<STORAGE>): Promise<IEvaSession<STORE, STORAGE>>;
    /**
     * Retrieves a session from the pool by it's session identifier
     *
     * @param {string} sessionId
     * @returns {(IEvaSession|undefined)}
     */
    getSession(sessionId: string): IEvaSession<STORE, STORAGE> | undefined;
}
//# sourceMappingURL=manager.d.ts.map