import springtree_solutionsEvaSuiteUiEvaLogin from '@bit/springtree_solutions.eva-suite-ui.eva-login';
import { FirebaseOptions } from '@firebase/app-types';
import { IEvaReduxStoreDefaultShape } from '@springtree/eva-sdk-redux-store-builder';
import { EvaStorage } from '@springtree/eva-sdk-core-storage';
import { Reducer } from 'redux';
import { Logic } from 'redux-logic';
import { IEvaSuiteStorage } from '../utils/storage';
import { IEvaServiceDefinition } from '@springtree/eva-services-core';
export interface IEvaContextBuilderOptions<STORAGE extends IEvaSuiteStorage> {
    /**
     * The calling application name which needs to be provided for the user agent header
     *
     * @type {string}
     */
    appName: string;
    /**
     * The calling application version which needs to be provided for the user agent header
     *
     * @type {string}
     */
    appVersion: string;
    /**
     * The EVA SDK based storage provider
     *
     * @type {EvaStorage<STORAGE>}
     */
    storage?: EvaStorage<STORAGE>;
    /**
     * The customer manager based login path requires Firebase configuration details
     *
     * @type {FirebaseOptions}
     */
    firebaseConfig: FirebaseOptions;
    /**
     * The instance of the login widget to use
     * Needs to be provided externally so it can be styled and integrated as the
     * calling app requires
     *
     * @type {springtree_solutionsEvaSuiteUiEvaLogin}
     */
    loginWidget: springtree_solutionsEvaSuiteUiEvaLogin;
}
export declare class EvaContextBuilder<STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage> {
    private options;
    /**
     * The collection of redux logics to build the store with
     *
     * @private
     * @type {Logic<STORE>[]}
     */
    private logic;
    /**
     * The collection of reducers to build the store with
     *
     * @private
     * @type {ReducersMapObject<Partial<STORE>>}
     */
    private reducer;
    /**
     * Creates an instance of EvaContextBuilder
     * Provide the template classes for your storage and store state interfaces
     *
     * @param {IEvaContextBuilderOptions<STORAGE>} options The options needed to create the store
     */
    constructor(options: IEvaContextBuilderOptions<STORAGE>);
    /**
     * Purge all previously set reducers and logics
     *
     */
    resetReducerAndLogic(): void;
    /**
     * Helper method to add both the reducer and logic for an EVA service in one call
     *
     * @template SVC
     * @param {keyof STORE} path
     * @param {new () => SVC} service
     */
    addEvaService<SVC extends IEvaServiceDefinition>(path: keyof STORE, service: new () => SVC): void;
    /**
     * Add a custom state reducer to the store
     *
     * @param {keyof STORE} path
     * @param {Reducer<STORE[keyof STORE]>} reducer
     */
    addReducer(path: keyof STORE, reducer: Reducer<Partial<STORE>[keyof STORE]>): void;
    /**
     * Add custom store logic
     *
     * @param {Logic<STORE>} logic The redux logic that is to be added to the store
     */
    addLogic(logic: Logic<STORE>[] | Logic<STORE>[]): void;
    /**
     * Create a Redux store using the provided and previously configured reducers and logics
     *
     */
    createStore(): Promise<import("redux").Store<STORE, import("redux").AnyAction>>;
}
//# sourceMappingURL=bootstrap.d.ts.map