import { filter, map, distinctUntilChanged } from 'rxjs/operators';
import { get, isNil, isEqual } from 'lodash';
/**
 * Helper method to create an observable for a path within the Redux store
 * Will only trigger when value set and has changed since previous emit
 *
 * @export
 * @template SVC The service the path points to
 * @template STORE The store shape to pull the service from
 * @param {Observable<STORE>} store$ The observable store data
 * @param {string} path The path to the service
 * @returns {Observable<SVC>}
 */
export function createObservableForPath(store$, path) {
    return store$.pipe(
    // tap( value => logger.debug('[EVA-CONTEXT]: State update', value)),
    map((value) => get(value, path)), filter(value => !isNil(value)), distinctUntilChanged(isEqual));
}
//# sourceMappingURL=pathObservable.js.map