import { Observable } from 'rxjs';
import { IEvaReduxStoreDefaultShape, IEvaServiceRequestAction, IEvaServiceResetAction, IEvaServiceResponseAction, IEvaServiceState } from '@springtree/eva-sdk-redux-store-builder';
import { IEvaServiceDefinition } from '@springtree/eva-services-core/lib/eva-services-core';
import { IEvaSession } from '../session';
import { IEvaSuiteStorage } from './storage';
import { IEvaServiceCallOptions } from '@springtree/eva-sdk-core-service';
/**
 * A service context provides a set if directly accessible observables and method
 * to access the data present for a service in the Redux store
 * Using the NonNullable helper here because the observable path builder filters
 * out empty values
 *
 * @export
 * @interface IEvaServiceContext
 * @template SVC
 */
export interface IEvaServiceContext<SVC extends IEvaServiceDefinition> {
    getState: () => IEvaServiceState<SVC>;
    getState$: () => Observable<IEvaServiceState<SVC>>;
    getRequest$: () => Observable<NonNullable<SVC['request']>>;
    getResponse$: () => Observable<NonNullable<SVC['response']>>;
    isFetching$: () => Observable<boolean>;
    getError$: () => Observable<any>;
    fetch: (payload?: SVC['request']) => IEvaServiceRequestAction<SVC>;
    response: (payload?: SVC['response']) => IEvaServiceResponseAction<SVC>;
    reset: (newState?: IEvaServiceState<SVC>) => IEvaServiceResetAction<SVC>;
    call: (payload?: SVC['request'], options?: IEvaServiceCallOptions) => Promise<SVC['response']>;
}
/**
 * The parameters needed to create a service context
 *
 * @export
 * @interface IEvaServiceContextParameters
 * @template SVC
 */
export interface IEvaServiceContextParameters<SVC extends IEvaServiceDefinition, STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage> {
    service: new () => SVC;
    path: keyof STORE;
    session: IEvaSession<STORE, STORAGE>;
    defaultPayload?: SVC['request'];
}
/**
 * Helper method to create an EVA Service context based on the Redux store
 *
 * @export
 * @template SVC
 * @param {keyof IEvaAdminStore} path
 * @param {IEvaSession} session
 * @returns {IEvaServiceContext<SVC>}
 */
export declare function createServiceContext<SVC extends IEvaServiceDefinition, STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage>(params: IEvaServiceContextParameters<SVC, STORE, STORAGE>): IEvaServiceContext<SVC>;
//# sourceMappingURL=serviceContext.d.ts.map